#!/usr/bin//python
# -*- coding: UTF -*-

#########################################################
#   Program pro ovladani osciloskopu UT2XXX             #
#                                                       #
#   Ing. Košan Tomáš, prosinec 2008                     #
#                                                       #
#   Licence: GNU GPL v2                                 #
#########################################################

from ut2XXX_definitions import *

import usb, time

class UNI_T_DSO:

	# init
	def __init__(self):
		
		# init of basic data structures
		self.ch1_data = {}
		self.ch2_data = {}
		self.status = {}
		
		# init status
		self.status["far_mode"]  = False
		self.status["locked"]    = False 
		self.status["info_data"] = []
		
		# init ch1 data
		self.ch1_data["samples"] = []
		self.ch1_data["V_div"]   = 0
		self.ch1_data["s_div"]   = 0
		self.ch1_data["couple"]  =""
		self.ch1_data["active"]  = False
		self.ch1_data["x_offset"]  = 0
		self.ch1_data["Bw_limit"] = False

		# init ch1 data
		self.ch2_data["samples"] = []
		self.ch2_data["V_div"]   = 0
		self.ch2_data["s_div"]   = 0
		self.ch2_data["couple"]  =""
		self.ch2_data["active"]  = False
		self.ch2_data["x_offset"]  = 0
		self.ch1_data["Bw_limit"] = False
		
		self.data_old = ()
		
		self.init()
		
	def init(self):	
		self.device = None
		self.interface_id = 0
		self.endpoints = []
		self.find_device()
		if not self.device:
			print "Error finding device. Exiting."
			self.is_present = False
		else:
			self.is_present = True
			self.handle = self.device.open()
			self.product = self.handle.getString(self.device.iProduct, 50),
			self.manufacturer = self.handle.getString(self.device.iManufacturer, 50)
			self.handle.claimInterface(self.interface_id)
			
			self.init_device()
		
		self.data = None
		
	# try to find proper USB device and get her config
	def find_device (self):
	
		busses = usb.busses()
		
		for bus in busses:
			devices = bus.devices
			for dev in devices:
				# is device in our PID and GID lists ?
				if dev.idProduct in PID and dev.idVendor in VID:
					print "Found UNI-T DSO on USB:"
					#print "  iManuf. :",dev.iManufacturer
					print "  idVendor:",dev.idVendor
					print "  idProduct:",dev.idProduct
					#return dev
					self.device = dev
					#print dev
					for config in dev.configurations:
						for intf in config.interfaces:
							#print "    Interface:",intf[0].interfaceNumber
							for alt in intf:
							#	print "    Alternate Setting:",alt.alternateSetting
							#	print "      Interface class:",alt.interfaceClass
							#	print "      Interface sub class:",alt.interfaceSubClass
							#	print "      Interface protocol:",alt.interfaceProtocol
								# find all endpoints
								for ep in alt.endpoints:
									self.endpoints.append(ep.address)
							#		print "      Endpoint:",hex(ep.address)
							#		print "        Type:",ep.type
							#		print "        Max packet size:",ep.maxPacketSize
							#		print "        Interval:",ep.interval
					
  	# enters far mode - it means all control on osciloscope is blocked
	def enter_far_mode(self):
		try:
			self.handle.controlMsg(0x42,0xb1,None,value=0xf0)
		except:
			print "Error entering remote control mode."
		else:
			self.status["far_mode"] = True			
		
	# leaves far mode - unblocks local control of osciloscope
	def leave_far_mode(self):
		try:
			self.handle.controlMsg(0x42,0xb1,None,value=0xf1)
		except:
			print "Error leaving remote control mode."
		else:
			self.status["far_mode"] = False			
			
	
	def get_info_from_device(self):		
		# asi nastaveni jednotlivych ovl. prvku
		# maybe state of control
		self.handle.controlMsg(0x42,0xb1,None, value=0x2c)
		self.handle.controlMsg(0x00,0xb2,None)
		ans = self.handle.controlMsg(0xc2,0xb2, 8)
		self.status["info_data"] = ans
	
	def send_message(self, message):
		# prvni funkcni test
		if not self.status["far_mode"]:
			print self.handle.controlMsg(0x42,0xb1,None,value=0xf0)
		
		# prvni funkcni test
		print self.handle.controlMsg(0x42,0xb1,None,value=message)
		
		time.sleep(0.05)
		
		if not self.status["locked"]:
			print self.handle.controlMsg(0x42,0xb1,None, value=0xf1)	
		
	def get_data(self):
		for i in range(0,5):
			self.handle.controlMsg(0x42,0xb1,None,index=0, value=0xe1)	
			time.sleep(0.01)	
			self.handle.controlMsg(0x42,0xb0,None,index=2, value=0x01)	
			time.sleep(0.02)	
			try:
				data = self.handle.bulkRead(130,1024,60)
			except:
				#print "Exception:",s
				data = self.data
			else:
				break
		#time.sleep(0.01)		
		self.handle.controlMsg(0x42,0xb0,None,index=0, value=0xdc)	
#		time.sleep(0.01)	
#		self.handle.controlMsg(0x42,0xb1,None,index=0, value=0xcc)	
		return data
		
	def get_parameters(self):
		
		self.handle.controlMsg(0x42,0xb0,None,index=0, value=0xdc)	
		time.sleep(0.02)	
		self.handle.controlMsg(0x42,0xb1,None,index=0, value=0xcc)	
		time.sleep(0.02)	
		self.handle.controlMsg(0x42,0xb1,None,index=0, value=0xcc)	
		time.sleep(0.02)	
		self.handle.controlMsg(0x42,0xb1,None,index=0, value=0xcc)	
		time.sleep(0.02)	
		self.handle.controlMsg(0x42,0xb1,None,index=0, value=0xcc)	
		time.sleep(0.02)	
		self.handle.controlMsg(0x42,0xb1,None,index=0, value=0xcc)	
		time.sleep(0.02)	
		self.handle.controlMsg(0x42,0xb1,None,index=0, value=0xe3)	
		time.sleep(0.02)	
		self.handle.controlMsg(0x42,0xb0,None)	
#		time.sleep(0.02)	
		try:
			data = self.handle.bulkRead(0x82,512,500)
		except:
#			try:
#				data = self.handle.bulkRead(0x82,512,200)
#			except:
			print "Exception, sending old data."
			self.handle.controlMsg(0x42,0xb0,None,index=0, value=0xdc)
			return self.data		
		
		self.handle.controlMsg(0x42,0xb0,None,index=0, value=0xdc)
		return data	

	def ping(self):
		self.handle.controlMsg(0x42,0xb1,None,index=0, value=0xcc)	
		time.sleep(0.01)	

	
	def close(self):
		print "Closing, bye."
		self.leave_far_mode()
		self.handle.releaseInterface()

				
	def init_device(self):
		try:
			print self.handle.controlMsg(0xc2,0xb2,8,index=4, value=0x08, timeout=300)	
			time.sleep(0.02)	
		except:
			pass
		self.handle.controlMsg(0x42,0xb1,None,index=0, value=0x2c)	
		time.sleep(0.02)	
		self.handle.controlMsg(0x42,0xb1,None,index=0, value=0xdc)	
		time.sleep(0.02)	
		self.handle.controlMsg(0x42,0xb1,None,index=0, value=0xcc)	
		time.sleep(0.02)	
		self.handle.controlMsg(0x42,0xb1,None,index=0, value=0xcc)	
		time.sleep(0.02)
		self.leave_far_mode()	
		
	
	def print_status(self):
		print "Device:",self.product[0]
		print "Manufacturer:", self.manufacturer
		self.get_waveform()
		print "------------------------------"
		if self.ch1_data["active"]:
			print "Chanel 1: ON"
		else:
			print "Chanel 1: OFF" 
		print " X range: ", self.ch1_data["V_div"],"V/div"
		print " Coupling:", self.ch1_data["couple"]
		print " Y range: ", self.ch1_data["s_div"], "s/div"
		
		if self.ch2_data["active"]:
			print "Chanel 2: ON"
		else:
			print "Chanel 2: OFF" 
		print " X range: ", self.ch2_data["V_div"],"V/div"
		print " Coupling:", self.ch2_data["couple"]
		print " Y range: ", self.ch2_data["s_div"], "s/div"
		

	# get samples from DSO
	def get_waveform(self):
		data = self.get_data()
		# check for proper length of data packet	
		if not data == None and len(data) == 1024:
			# compute V/div for each chanel
			self.ch1_data["V_div"] = X_RANGE[data[X_SENSE_CH1]]*(10**(data[X_PROBE_CH1]))
			self.ch2_data["V_div"] = X_RANGE[data[X_SENSE_CH2]]*(10**(data[X_PROBE_CH2]))
			# check for coupling
			self.ch1_data["couple"] = COUPLING[data[COUPLING_CH1]]
			self.ch2_data["couple"] = COUPLING[data[COUPLING_CH2]]
			# save samples data to buffers
			self.ch1_data["samples"] = data[516:766]
			self.ch2_data["samples"] = data[770:1020]
			# compute s/div
			self.ch1_data["s_div"] = Y_RANGE[data[Y_SCALE_CH1]]
			self.ch2_data["s_div"] = Y_RANGE[data[Y_SCALE_CH2]]
			# check status of channels
			self.ch1_data["active"] = bool(data[CHANNEL_STATE] & 0x01)
			self.ch2_data["active"] = bool(data[CHANNEL_STATE] & 0x02)
			# x offset
			self.ch1_data["x_offset"] = 0x7e - data[X_POS_CH1]
			#print self.ch1_x_offset
			self.ch2_data["x_offset"] = 0x7e - data[X_POS_CH2]
			
			self.ch1_data["Bw_limit"] = bool(data[BW_LIMIT_CH1])
			self.ch2_data["Bw_limit"] = bool(data[BW_LIMIT_CH2])
		else:
			print time.time(),"Data buffer error"
		
	def test_screenshot(self):
		self.get_screenshot()
		for i in range(0,len(self.data)):
			try:
				if self.data_old[i] != self.data[i]:
					print "Zmena na", i 
			except:
				pass
		self.data_old = self.data
		
	def get_screenshot(self):
		self.data = ()
		self.handle.controlMsg(0x42,0xb1,None,index=0, value=0xe2)	
		time.sleep(0.01)	
		self.handle.controlMsg(0x42,0xb0,None,index=2, value=0x26)	
		time.sleep(0.02)
		for i in range(0,5):
			time.sleep(0.05)
			print i	
			try:
				if i == 4:
					data = self.handle.bulkRead(130,6144,1000)
				else:	
					data = self.handle.bulkRead(130,8192,1000)
			except:
				print "ERR:"
				
			self.data += data
	
		print len(self.data)
		self.write_pixmap()
		
	def write_pixmap(self):
		
		width=320; height=240
		comment='any comment string'
		ftype='P6' #use 'P3' for ascii, 'P6' for binary
		
		ppmfile=open('testimage.ppm','wb')
		ppmfile.write("%s\n" % (ftype)) 
		ppmfile.write("#%s\n" % comment ) 
		ppmfile.write("%d %d\n" % (width, height)) 
		ppmfile.write("32\n")
		
#		if ftype=='P3':
#		for red,green,blue in triplets:
#			ppmfile.write("%d %d %d\n" % (red,green,blue)) 
#		elif ftype=='P6': #print 1 byte per color
		for index in range(0,320*120):
			color = self.data[index]
			color1 = (color & 0x0F) + (color & 0x0F) * 16
			color2 = (color & 0xF0) + (color & 0xF0) / 16
			#color1 = color & 0x0F
			#color2 = color & 0xF0 / 16
			
			#print hex(color), ' -> ', hex(color1), hex(color2)
			
			ppmfile.write("%c%c%c" % (color1, color1, color1))
			ppmfile.write("%c%c%c" % (color2, color2, color2))
		
		ppmfile.close()


		#return (str(self.data).replace(",","").replace("(","").replace(")",""))

if __name__ == '__main__':

	dso = UNI_T_DSO()
	if dso.is_present:
		dso.get_info_from_device()
#		while raw_input("q = konec") != "q":
#			dso.test_screenshot()
#		dso.get_screenshot()
		dso.print_status()
		dso.close()
	else:
		print "Device was not recognized or found."
