#!/usr/bin/env python
# -*- coding: UTF -*-

#############################################################################
# qt2062 - PyQT application template for KDevelop
#
# Translated from C++ qmakeapp.cpp
# (qmakeapp.cpp - Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.)
#
# This file is part of an example program for Qt.  This example
# program may be used, distributed and modified without limitation.
#
#############################################################################

import sys, time
import os.path
from qt import *
from qtcanvas import *

import threading, Queue

import UT2XXX
import utils
import graphic_classes
	
# from main to thread
Que_main2thread = Queue.Queue()
Que_thread2main = Queue.Queue()
	
	
def DSO_thread():
	
	want_run = True
	
	# basic init of DSO comunication
	dso = UT2XXX.UNI_T_DSO()
	# test if device is connected
	if not dso.is_present:
		Que_thread2main.put("ERR:Device was not found.")
		return
	
	msg = ""
	
	while want_run:
		try:
			msg = Que_main2thread.get_nowait()
		except:
			pass
		else:
			if msg == "END_NOW":
				want_run = False
				
			elif msg == "GET_WAVE":
				#print "Msg form main: get wave"
				dso.get_waveform()
				Que_thread2main.put("DATA")
				Que_thread2main.put(dso.ch1_data)	
				Que_thread2main.put(dso.ch2_data)
				#print "Data send."
			else:
				msg = ""
		dso.ping()	
		#time.sleep(0.01)	

	print "Thread end."

class DSO_main(QMainWindow):

	def __init__(self):
	        QMainWindow.__init__(self, None, u"mainWindow")
		self.layout = QGridLayout(self, 2, 2, 10, 0, "Layout")
		self.buttons_layout = QVBoxLayout(self, 3, 0, "Btn layout") 
		self.layout.addLayout(self.buttons_layout, 0, 1)
		
		# button configuration
		
		# create buttons
		self.btClose = QPushButton(self, "btClose")
		self.btClose.setText(u"Uzavři")
	
		self.btRefresh = QPushButton(self, "btRefresh")
		self.btRefresh.setText(u"Obnov")
		
		self.btAutoRefresh = QPushButton(self, "btAutoRefresh")
		self.btAutoRefresh.setText(u"Obnovuj periodicky - vypnuto")
		
		self.btSave = QPushButton(self, "btSave")
		self.btSave.setText(u"Ulož screenshot")
		
				
		# add to layout
		self.buttons_layout.addWidget(self.btClose, 0, Qt.AlignCenter)
		self.buttons_layout.addWidget(self.btRefresh, 1, Qt.AlignCenter)
		self.buttons_layout.addWidget(self.btAutoRefresh, 2, Qt.AlignCenter)
		self.buttons_layout.addWidget(self.btSave, 3, Qt.AlignCenter)
		
		# connect signals
		self.connect(self.btClose, SIGNAL("clicked()"), self.close)
		self.connect(self.btRefresh, SIGNAL("clicked()"), self.refresh)
		self.connect(self.btAutoRefresh, SIGNAL("clicked()"), self.auto_refresh)
		self.connect(self.btSave, SIGNAL("clicked()"), self.save_screen)
	
		# graphic view configuration
		
		#create graphic window
		self.color = QColor()
		
		self.canvas = QCanvas(self, "canvas")
		self.canvas.resize(521,421)
		self.color.setNamedColor("black")
		self.canvas.setBackgroundColor(self.color)
		self.canvas.update()
		
		self.view = QCanvasView(self, "view")
		self.view.setCanvas(self.canvas)
		
		self.layout.addWidget(self.view,0,0, Qt.AlignCenter | Qt.AlignTop)
		self.layout.setResizeMode(QLayout.FreeResize)
		
		self.paint_grid()
		
		# setup of text labels
		self.param_layout = QGridLayout(self,4,2)
		
		# chanels
		self.param_layout.addWidget(QLabel(u"Kanál CH1:",self), 0, 0, Qt.AlignLeft | Qt.AlignTop)
		
		self.CH1_Vdiv = QLabel(u"<b>----------</b>",self)
		self.param_layout.addWidget(self.CH1_Vdiv, 1, 0, Qt.AlignLeft | Qt.AlignTop)
		
		self.param_layout.addWidget(QLabel(u"Kanál CH2:",self), 2, 0, Qt.AlignLeft | Qt.AlignTop)
		
		self.CH2_Vdiv = QLabel(u"<b>----------</b>",self)
		self.param_layout.addWidget(self.CH2_Vdiv, 3, 0, Qt.AlignLeft | Qt.AlignTop)
		
		# triger and time base
		self.param_layout.addWidget(QLabel(u"Časová základna:",self), 0, 1, Qt.AlignLeft | Qt.AlignTop)
		
		self.timebase = QLabel(u"<b>---</b>",self)
		self.param_layout.addWidget(self.timebase, 1, 1, Qt.AlignLeft | Qt.AlignTop)
		
		self.param_layout.addWidget(QLabel(u"Spouštění:",self), 2, 1, Qt.AlignLeft | Qt.AlignTop)
		
		self.trigger = QLabel(u"<b>----</b>",self)
		self.param_layout.addWidget(self.trigger, 3, 1, Qt.AlignLeft | Qt.AlignTop)
		
		self.layout.addLayout(self.param_layout,1,0)
		
		self.ch1_points = []
		self.ch1_mark = graphic_classes.Cursor(self.canvas, 1)
		self.ch2_points = []
		self.ch2_mark = graphic_classes.Cursor(self.canvas, 2)
		
		self.ch3_mark = graphic_classes.Cursor(self.canvas, 3)
		
		self.dso_thread = threading.Thread(target=DSO_thread)
		self.dso_thread.start()
		
		# timer for checking msg queue
		self.timer = QTimer(self, "timer")
		self.timer.start(10)
		self.connect(self.timer, SIGNAL("timeout()"), self.update_state)
		
		# timer for auto refresh
		self.autoupdate_timer = QTimer(self, "timer")
		self.connect(self.autoupdate_timer, SIGNAL("timeout()"), self.refresh)




	def update_state(self):
		#print "Timer start"
		try:
			msg = Que_thread2main.get_nowait()
		except Exception:
			pass
		else:
#			print "Msg from thread:",msg
			if msg == "DATA":
				self.ch1_data = Que_thread2main.get()
				self.ch2_data = Que_thread2main.get()
				self.paint_wave()
			
			if msg.find("ERR:")>-1:
				QMessageBox.critical(self, u"Chyba", u"Zařízení nenalezeno. Tato chyba je kritická a program se ukončí.\nUjistěte se, že máte digitální osciloskop zapojený do USB a zapnutý.")
				#self.close()
				
	def paint_wave(self):
		
		# channel 1
		pen = QPen()
		self.color.setNamedColor("cyan")
		pen.setColor(self.color)
		
		brush = QBrush(self.color)
		
		
		if self.ch1_data["active"]:
			self.ch1_mark.show()
			x = 20
			y = 20
			index = 0
			
			self.ch1_mark.setY(2*(106-self.ch1_data["x_offset"]))
			
			while len(self.ch1_points) < len(self.ch1_data["samples"]):
				self.ch1_points.append(QCanvasLine(self.canvas))
				
		
			for sample in self.ch1_data["samples"]:
				self.point = self.ch1_points[index]
				index += 1 
				# first point
				if x == 20:
					self.point.setPoints(x,(2*(255 - sample) - 34),x,(2*(255 - sample) - 34))
				else:
					self.point.setPoints(x-2,y,x,(2*(255 - sample) - 34))
					
				x = x + 2
				y = (2*(255 - sample) - 34)
				self.point.setPen(pen)
				self.point.show()
			
		else:
			for point in self.ch1_points:
				point.hide()
			self.ch1_mark.hide()
			
		self.color.setNamedColor("yellow")
		
		
		if self.ch2_data["active"]:
			
			self.ch2_mark.show()
			
			pen.setColor(self.color)
			
			self.ch2_mark.setY(2*(106-self.ch2_data["x_offset"]))
		
			x = 20
			y = 20
			index = 0
			
			# init of all samples of CH2
			while len(self.ch2_points) < len(self.ch2_data["samples"]):
				self.ch2_points.append(QCanvasLine(self.canvas))
			
			# applying samples to each point
			for sample in self.ch2_data["samples"]:
				self.point = self.ch2_points[index]
				index += 1 
				# first point
				if x == 20:
					self.point.setPoints(x,(2*(255 - sample) - 34),x,(2*(255 - sample) - 34))
				else:
					self.point.setPoints(x-2,y,x,(2*(255 - sample) - 34))
					
				x = x + 2
				y = (2*(255 - sample) - 34)
				self.point.setPen(pen)
				self.point.show()
		else:
			for point in self.ch2_points:
				point.hide()
			self.ch2_mark.hide()
			
		# repaint display
		self.canvas.update()
		self.view.update()
		
		# refresh text data
		if self.ch1_data["Bw_limit"]:
			self.CH1_Vdiv.setText(str(self.ch1_data["V_div"]) +u"V/div Bw=20MHz")
		else:
			self.CH1_Vdiv.setText(str(self.ch1_data["V_div"]) +u"V/div")
		
		if self.ch2_data["Bw_limit"]:	
			self.CH2_Vdiv.setText(str(self.ch2_data["V_div"]) +u"V/div Bw=20MHz")
		else:
			self.CH2_Vdiv.setText(str(self.ch2_data["V_div"]) +u"V/div")
			
		self.timebase.setText(u"<b>"+utils.float2engstr(self.ch1_data["s_div"])+"s/div</b>")
		
		#self.trigger.setText()
		
	 
	# paints grid on main view
	def paint_grid(self):
		pen = QPen()
		self.color.setNamedColor("white")
		pen.setColor(self.color)
		ts = time.time()
		
		# we will generate DSO screen grid
		for x in xrange(20,540,10):
			for y in xrange(20,520,50):
				
				self.point = QCanvasLine(self.canvas)
				
				if y == 20:
					self.point.setPoints(x,y,x,y+3)
				elif y == 220:
					self.point.setPoints(x,y-2,x,y+2)
				elif y == 420:
					self.point.setPoints(x,y,x,y-3)
				else:
					self.point.setPoints(x,y,x,y)
				
				self.point.setPen(pen)
				self.point.show()

		for x in xrange(20,521,50):
			for y in xrange(20,421,10):
				
				self.point = QCanvasLine(self.canvas)
				
				if x == 0:
					self.point.setPoints(x,y,x+3,y)
				elif x == 270:
					self.point.setPoints(x-2,y,x+2,y)
				elif x == 520:
					self.point.setPoints(x,y,x-3,y)
				else:
					self.point.setPoints(x,y,x,y)
				
				self.point.setPen(pen)
				self.point.show()

			
		self.canvas.update()
		self.view.update()
		print "Repaint in",time.time() - ts, "s"


	def refresh(self):
		if Que_main2thread.empty():
			Que_main2thread.put("GET_WAVE")
		
	def auto_refresh(self):
		if self.autoupdate_timer.isActive():
			self.autoupdate_timer.stop()
			self.btAutoRefresh.setText(u"Obnovuj periodicky - vypnuto")
		else:	
			self.autoupdate_timer.start(100)	
			self.btAutoRefresh.setText(u"Obnovuj periodicky - zapnuto")
		
	def closeEvent(self, closeEvent):
		print "Closing"
		Que_main2thread.put("END_NOW")
		closeEvent.accept()
		return
	
	#void Main::print()
	#{
	#if ( !printer ) printer = new QPrinter;
	#if ( printer->setup(this) ) {
		#QPainter pp(printer);
		#canvas.drawArea(QRect(0,0,canvas.width(),canvas.height()),&pp,FALSE);
	#}
	#}	
	def save_screen(self):
		screenshot = QPixmap(800,600,24)
		self.canvas.drawArea(self.canvas.rect(), QPainter(screenshot))
		filename = QFileDialog.getSaveFileName(u"./", u"Obrázky (*.png)", self, u"???", u"Zadejte jméno a cestu souboru")
		if filename:
			screenshot.convertToImage().save(filename,"PNG")
		

def main(args):
    app=QApplication(args)
    mainWindow = DSO_main()
    mainWindow.setFixedSize(QSize(800,600))
    mainWindow.setMinimumSize(QSize(800,600))
    mainWindow.show()
    app.connect(app, SIGNAL("lastWindowClosed()"), app, SLOT("quit()"))
    app.exec_loop()



if __name__ == "__main__":
    main(sys.argv)
